import re

import numpy
import pandas as pd
import tabula

from .base import AllergenState, BaseParser
from .util import removeNewLine

REMOTE_URL = "http://www3.akindo-sushiro.co.jp/pdf/menu/allergy.pdf"
BASE_LAYOUT = [
    ("卵", "egg", True),  # header, key, required
    ("乳", "milk", True),
    ("小麦", "wheat", True),
    ("えび", "shrimp", True),
    ("かに", "crab", True),
    ("そば", "buckwheat", True),
    ("落花生", "peanut", True),
    ("あわび", "abalone", True),
    ("いか", "squid", True),
    ("いくら", "salmonRoe", True),
    ("オレンジ", "orange", True),
    ("キウイ", "kiwi", True),
    ("牛肉", "beef", True),
    ("クルミ", "walnut", True),
    ("鮭", "salmon", True),
    ("さば", "mackerel", True),
    ("大豆", "soybean", True),
    ("鶏肉", "chicken", True),
    ("バナナ", "banana", True),
    ("豚肉", "pork", True),
    ("松茸", "matsutakeMushroom", True),
    ("桃", "peach", True),
    ("山芋", "yamOrSweetPotato", True),
    ("りんご", "apple", True),
    ("ゼラチン", "gelatin", True),
    ("ごま", "sesame", True),
    ("カシューナッツ", "cashewNut", True),
]
TABLE_LAYOUT = [
    ("分類", "category", True),
    ("商品名", "name", True),
    *BASE_LAYOUT,
    ("なし", "none", False),
]


class SushiroAllergyParser(BaseParser):
    def parse(self, filename: str = REMOTE_URL) -> pd.DataFrame:
        df = self._parse_table(filename, TABLE_LAYOUT)

        for _, key, _ in BASE_LAYOUT:
            df[key] = df[key].map(self.allergen_state_of)

        return df

    def _parse_table(self, filename, layout) -> pd.DataFrame:

        df = tabula.read_pdf(filename, pages="all", lattice=True)

        columns = [re.sub(r"\s", "", column) for column in df.columns]
        expected_columns = [column[0] for column in layout]
        assert columns == expected_columns, f"{filename} のテーブルレイアウトが変更されました。"

        df.columns = [column[1] for column in layout]

        # Remove unnecessary columns
        for _, key, required in layout:
            if not required:
                del df[key]

        df_filterd = df.query('category != "分類"')

        df_cleansed = df_filterd.applymap(removeNewLine)

        return df_cleansed

    def allergen_state_of(self, value: str) -> AllergenState:
        if value == "●":
            return AllergenState.CONTAIN
        elif value == "○":
            return AllergenState.MAY_CONTAIN
        elif numpy.isnan(value):
            return AllergenState.NOT_CONTAIN
        else:
            raise ValueError(f"Unknown AllergenState {value}.")




from abc import ABCMeta, abstractmethod
from enum import Enum, auto

from pandas import DataFrame


class AllergenState(Enum):
    CONTAIN = auto()  # 含む
    MAY_CONTAIN = auto()  # 製造工程で混入する可能性あり
    NOT_CONTAIN = auto()  # 含まない

    def __str__(self):
        return self.name


class BaseParser(metaclass=ABCMeta):
    @abstractmethod
    def parse(self, filename: str) -> DataFrame:
        raise NotImplementedError

    @abstractmethod
    def allergen_state_of(self, value: str) -> AllergenState:
        raise NotImplementedError



import re

import numpy
import pandas as pd
import tabula

from .base import AllergenState, BaseParser
from .util import removeNewLine

REMOTE_URL = "http://www.kura-corpo.co.jp/common/pdf/kura_allergen.pdf"
BASE_LAYOUT = [
    ("卵", "egg", True),  # header, key, required
    ("小麦", "wheat", True),
    ("乳・乳製品", "milk", True),
    ("落花生ピーナッツ", "peanut", True),
    ("ソバ", "buckwheat", True),
    ("えび", "shrimp", True),
    ("かに", "crab", True),
    ("いか", "squid", True),
    ("いくら", "salmonRoe", True),
    ("さけ", "salmon", True),
    ("さば", "mackerel", True),
    ("牛肉", "beef", True),
    ("鶏肉", "chicken", True),
    ("豚肉", "pork", True),
    ("大豆", "soybean", True),
    ("オレンジ", "orange", True),
    ("りんご", "apple", True),
    ("ゼラチン", "gelatin", True),
    ("クルミ", "walnut", True),
    ("バナナ", "banana", True),
    ("キウイ", "kiwi", True),
    ("山芋", "yamOrSweetPotato", True),
    ("モモ", "peach", True),
    ("ごま", "sesame", True),
    ("あわび", "abalone", True),
    ("カシューナッツ", "cashewNut", True),
    ("松茸", "matsutakeMushroom", True),
]
PAGE1_LAYOUT = [
    ("品名", "name", True),
    ("(kカ一cロ皿aリ当lーり)", "calory", False),
    ("該当なし", "none", False),
    *BASE_LAYOUT,
]
PAGE2_LAYOUT = PAGE1_LAYOUT
PAGE3_LAYOUT = [
    ("品名", "name", True),
    ("一(皿kカ・cロ一aリlー杯当)り", "calory", False),
    ("該当なし", "none", False),
    *BASE_LAYOUT,
]
PAGE4_LAYOUT = [("品名", "name", True), ("該当なし", "none", False), *BASE_LAYOUT]
PAGES = [
    ("1", "定番寿司", PAGE1_LAYOUT),
    ("2", "限定商品", PAGE2_LAYOUT),
    ("3", "サイドメニュー", PAGE3_LAYOUT),
    ("4", "その他", PAGE4_LAYOUT),
]


class KuraAllergyParser(BaseParser):
    def parse(self, filename: str = REMOTE_URL) -> pd.DataFrame:
        df_list = []
        for page, category, layout in PAGES:
            df = self._parse_table(filename, page, category, layout)
            df_list.append(df)

        df_all = pd.concat(df_list, ignore_index=True)

        for _, key, _ in BASE_LAYOUT:
            df_all[key] = df_all[key].map(self.allergen_state_of)

        return df_all

    def _parse_table(
        self, filename: str, page: int, category: str, layout: list
    ) -> pd.DataFrame:

        df = tabula.read_pdf(filename, pages=str(page), lattice=True)

        columns = [re.sub(r"\s", "", column) for column in df.columns]
        expected_columns = [column[0] for column in layout]
        assert columns == expected_columns, f"{filename} のテーブルレイアウトが変更されました。{page}ページ目"

        df.columns = [column[1] for column in layout]

        # Remove unnecessary columns
        for _, key, required in layout:
            if not required:
                del df[key]

        df_cleansed = df.applymap(removeNewLine)

        df_cleansed["category"] = category

        return df_cleansed

    def allergen_state_of(self, value: str) -> AllergenState:
        if value == "●":
            return AllergenState.CONTAIN
        elif value == "▲":
            return AllergenState.MAY_CONTAIN
        elif numpy.isnan(value):
            return AllergenState.NOT_CONTAIN
        else:
            raise ValueError(f"Unknown AllergenState {value}.")



# -*- coding: utf-8 -*-
from distutils.core import setup

package_dir = \
{'': 'src'}

packages = \
['sushi_allergy_parser']

package_data = \
{'': ['*']}

install_requires = \
['tabula-py>=1.4,<2.0']

setup_kwargs = {
    'name': 'sushi-allergy-parser',
    'version': '0.1.0',
    'description': '',
    'long_description': None,
    'author': 't3yamoto',
    'author_email': '3yamoto.dev@gmail.com',
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
